//
//  NSDictionary+SyncKitAdditions.m
//  SyncKit
//
//  Created by Logan Collins on 2/10/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import "NSDictionary+SyncKitAdditions.h"

#import "NSString+SyncKitAdditions.h"


@implementation NSDictionary (SyncKitAdditions)

- (NSDictionary *)snk_filteredDictionaryUsingBlock:(BOOL (^)(id key, id value, BOOL *stop))block {
    NSMutableDictionary *newDictionary = [NSMutableDictionary dictionaryWithCapacity:[self count]];
    BOOL stop = NO;
    for (id key in self) {
        id value = self[key];
        BOOL result = block(key, value, &stop);
        if (result) {
            newDictionary[key] = value;
        }
        if (stop) {
            break;
        }
    }
    return newDictionary;
}

- (NSData *)snk_HTTPBody {
    NSArray *keys = [[self allKeys] sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
    NSMutableArray *pairs = [NSMutableArray arrayWithCapacity:[keys count]];
    for (NSUInteger i=0; i<[keys count]; i++) {
        NSString *key = [keys objectAtIndex:i];
        NSString *value = [self objectForKey:key];
        [pairs addObject:[NSString stringWithFormat:@"%@=%@", key, value]];
    }
    return [[pairs componentsJoinedByString:@"&"] dataUsingEncoding:NSUTF8StringEncoding];
}

- (NSString *)snk_HTTPParametersString {
    NSArray *keys = [[self allKeys] sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
    NSMutableArray *pairs = [NSMutableArray arrayWithCapacity:[keys count]];
    for (NSUInteger i=0; i<[keys count]; i++) {
        NSString *key = [keys objectAtIndex:i];
        NSString *value = [self objectForKey:key];
        [pairs addObject:[NSString stringWithFormat:@"%@=%@", [key snk_URLEncodedString], [value snk_URLEncodedString]]];
    }
    return [pairs componentsJoinedByString:@"&"];
}

@end


@implementation NSMutableDictionary (SyncKitAdditions)

- (void)snk_filterUsingBlock:(BOOL (^)(id, id, BOOL *))block {
    NSMutableDictionary *newDictionary = [NSMutableDictionary dictionaryWithCapacity:[self count]];
    BOOL stop = NO;
    for (id key in self) {
        id value = self[key];
        BOOL result = block(key, value, &stop);
        if (result) {
            newDictionary[key] = value;
        }
        if (stop) {
            break;
        }
    }
    [self setDictionary:newDictionary];
}

@end
