//
//  NSEntityDescription+SyncKitAdditions.m
//  SyncKit
//
//  Created by Logan Collins on 4/15/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import "NSEntityDescription+SyncKitAdditions.h"

#import "SNKPersistentSyncCoordinator.h"
#import "NSPropertyDescription+SyncKitAdditions.h"
#import <objc/runtime.h>

#import "SNKChangeObject_Private.h"


@implementation NSEntityDescription (SyncKitAdditions)

static NSString * const SNKIsSyncableKey = @"com.sunflowersw.synckit.syncable";

- (NSNumber *)snk_recursiveSyncable {
    NSNumber *result = nil;
    id value = [[self userInfo] objectForKey:SNKEntityIsSyncableKey];
    if ([value isKindOfClass:[NSNumber class]]) {
        result = value;
    }
    else if ([value isKindOfClass:[NSString class]]) {
        if ([[value lowercaseString] isEqualToString:@"true"] || [[value lowercaseString] isEqualToString:@"yes"] || [[value lowercaseString] isEqualToString:@"1"]) {
            result = [NSNumber numberWithBool:YES];
        }
        else {
            result = [NSNumber numberWithBool:NO];
        }
    }
    
    if (result == nil) {
        // Only recurse if the result is nil, not false, as subentities can opt-out of syncing inherited from a superentity
        result = [[self superentity] snk_recursiveSyncable];
    }
    
    return result;
}

- (BOOL)snk_isSyncable {
    NSNumber *value = objc_getAssociatedObject(self, (__bridge void *)SNKIsSyncableKey);
    if (value == nil) {
        value = [self snk_recursiveSyncable];
        if (value == nil) {
            value = [NSNumber numberWithBool:NO];
        }
        objc_setAssociatedObject(self, (__bridge void *)SNKIsSyncableKey, value, OBJC_ASSOCIATION_COPY);
    }
    return [value boolValue];
}

static NSString * const SNKSyncableIdentifierAttributeNameKey = @"com.sunflowersw.synckit.identifier-attribute";

- (NSString *)snk_recursiveSyncableIdentifierAttributeName {
    NSDictionary *userInfo = [self userInfo];
    NSString *value = userInfo[SNKEntityIdentifierAttributeNameKey];
    if (value == nil) {
        value = [[self superentity] snk_recursiveSyncableIdentifierAttributeName];
    }
    return value;
}

- (NSString *)snk_syncableIdentifierAttributeName {
    NSString *attributeName = objc_getAssociatedObject(self, (__bridge void *)SNKSyncableIdentifierAttributeNameKey);
    if (attributeName == nil) {
        attributeName = [self snk_recursiveSyncableIdentifierAttributeName];
        if (attributeName == nil) {
            attributeName = @"identifier";
        }
        
        NSPropertyDescription *property = [[self propertiesByName] objectForKey:attributeName];
        if (property != nil && [property isKindOfClass:[NSAttributeDescription class]]) {
            if (![(NSAttributeDescription *)property attributeType] == NSStringAttributeType) {
                @throw [NSException exceptionWithName:NSInvalidArgumentException reason:[NSString stringWithFormat:@"Identifier property '%@' of entity '%@' is not a string attribute.", attributeName, self] userInfo:nil];
            }
        }
        else if (property != nil) {
            @throw [NSException exceptionWithName:NSInvalidArgumentException reason:[NSString stringWithFormat:@"Identifier property '%@' of entity '%@' is not a string attribute.", attributeName, self] userInfo:nil];
        }
        else {
            @throw [NSException exceptionWithName:NSInvalidArgumentException reason:[NSString stringWithFormat:@"Identifier property '%@' of entity '%@' is not defined.", attributeName, self] userInfo:nil];
        }
        
        objc_setAssociatedObject(self, (__bridge void *)SNKSyncableIdentifierAttributeNameKey, attributeName, OBJC_ASSOCIATION_COPY);
    }
    return attributeName;
}

static NSString * const SNKSyncablePropertiesByNameKey = @"SNKSyncablePropertiesByName";

- (NSDictionary *)snk_syncablePropertiesByName {
    NSDictionary *syncablePropertiesByName = nil;
    @synchronized(self) {
        syncablePropertiesByName = objc_getAssociatedObject(self, (__bridge void *)SNKSyncablePropertiesByNameKey);
        if (syncablePropertiesByName == nil) {
            NSDictionary *properties = [self propertiesByName];
            NSMutableDictionary *newSyncablePropertiesByName = [NSMutableDictionary dictionaryWithCapacity:[properties count]];
            
            for (NSString *name in properties) {
                NSPropertyDescription *property = properties[name];
                
                BOOL isSyncable = [property snk_isSyncable];
                if (isSyncable) {
                    NSString *syncableName = [property snk_syncableName];
                    newSyncablePropertiesByName[syncableName] = property;
                }
            }
            
            syncablePropertiesByName = [newSyncablePropertiesByName mutableCopy];
            objc_setAssociatedObject(self, (__bridge void *)SNKSyncablePropertiesByNameKey, syncablePropertiesByName, OBJC_ASSOCIATION_COPY);
        }
    }
    return syncablePropertiesByName;
}

@end
