//
//  NSPropertyDescription+SyncKitAdditions.m
//  SyncKit
//
//  Created by Logan Collins on 4/15/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import "NSPropertyDescription+SyncKitAdditions.h"

#import "SNKPersistentSyncCoordinator.h"
#import <objc/runtime.h>


@implementation NSPropertyDescription (SyncKitAdditions)

static NSString * const SNKIsSyncableKey = @"com.sunflowersw.synckit.syncable";

- (BOOL)snk_isSyncable {
    NSNumber *syncable = objc_getAssociatedObject(self, (__bridge void *)SNKIsSyncableKey);
    if (syncable == nil) {
        NSDictionary *userInfo = [self userInfo];
        
        // Determine if the property is syncable
        id value = [userInfo objectForKey:SNKPropertyIsSyncableKey];
        
        BOOL isSyncable = NO;
        if ([value isKindOfClass:[NSNumber class]]) {
            isSyncable = [value boolValue];
        }
        else if ([value isKindOfClass:[NSString class]]) {
            isSyncable = ([[value lowercaseString] isEqualToString:@"true"] || [[value lowercaseString] isEqualToString:@"yes"] || [value isEqualToString:@"1"]);
        }
        
        syncable = [NSNumber numberWithBool:isSyncable];
        objc_setAssociatedObject(self, (__bridge void *)SNKIsSyncableKey, syncable, OBJC_ASSOCIATION_COPY);
    }
    return [syncable boolValue];
}

static NSString * const SNKSyncableNameKey = @"com.sunflowersw.synckit.syncable-name";

- (NSString *)snk_syncableName {
    NSString *syncableName = objc_getAssociatedObject(self, (__bridge void *)SNKSyncableNameKey);
    if (syncableName == nil) {
        NSDictionary *userInfo = [self userInfo];
        
        // Determine if the property is syncable
        id value = [userInfo objectForKey:SNKSyncableNameKey];
        if (value == nil) {
            value = self.name;
        }
        
        syncableName = value;
        objc_setAssociatedObject(self, (__bridge void *)SNKSyncableNameKey, value, OBJC_ASSOCIATION_COPY);
    }
    return syncableName;
}

@end
