//
//  SNKChangeContext.h
//  SyncKit
//
//  Created by Logan Collins on 10/26/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>


@class SNKChange, SNKChangeEntity, SNKChangeCoordinator;


@interface SNKChangeContext : NSObject

- (id)initWithChangeCoordinator:(SNKChangeCoordinator *)coordinator;

@property (nonatomic, readonly) SNKChangeCoordinator *changeCoordinator;

// Operations
- (void)performBlock:(void (^)(void))block;
- (void)performBlockAndWait:(void (^)(void))block;

// Persistence
- (BOOL)commit:(NSError **)outError;
- (void)rollback;

// Entities
@property (nonatomic, copy, readonly) NSArray *entities;
- (SNKChangeEntity *)entityWithName:(NSString *)entityName create:(BOOL)create;
- (void)deleteEntity:(SNKChangeEntity *)entity;

// Changes
- (NSArray *)changesMatchingPredicate:(NSPredicate *)predicate;

- (void)insertedRecordWithIdentifier:(NSString *)recordIdentifier entity:(NSString *)entityName attributes:(NSSet *)attributes;
- (void)updatedRecordWithIdentifier:(NSString *)recordIdentifier entity:(NSString *)entityName attributes:(NSSet *)attributes;
- (void)deletedRecordWithIdentifier:(NSString *)recordIdentifier entity:(NSString *)entityName;

- (void)deleteChange:(SNKChange *)change;
- (void)deleteAllChangesForEntity:(SNKChangeEntity *)entity;
- (void)deleteAllChanges;

@end
