//
//  SNKChangeCoordinator.m
//  SyncKit
//
//  Created by Logan Collins on 10/24/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import "SNKChangeCoordinator.h"
#import "SNKChangeCoordinator_Private.h"


@interface SNKChangeCoordinator ()

@property (nonatomic, copy, readwrite) NSURL *URL;

@end


@implementation SNKChangeCoordinator {
    NSPersistentStoreCoordinator *_persistentStoreCoordinator;
}

+ (NSManagedObjectModel *)managedObjectModel {
    static NSManagedObjectModel *managedObjectModel = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        NSURL *managedObjectModelURL = [[NSBundle bundleForClass:[SNKChangeCoordinator class]] URLForResource:@"SyncKitChanges" withExtension:@"momd"];
        if (managedObjectModelURL == nil) {
            managedObjectModelURL = [[NSBundle mainBundle] URLForResource:@"SyncKitChanges" withExtension:@"momd"];
        }
        if (managedObjectModelURL == nil) {
            @throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Unable to find SyncKit changes model" userInfo:nil];
        }
        managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:managedObjectModelURL];
    });
    return managedObjectModel;
}

- (id)initWithURL:(NSURL *)aURL {
    NSParameterAssert(aURL != nil);
    
    self = [super init];
    if (self) {
        self.URL = aURL;
        
        _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[SNKChangeCoordinator managedObjectModel]];
        
        NSError *error = nil;
        NSPersistentStore *store = [_persistentStoreCoordinator addPersistentStoreWithType:NSBinaryStoreType configuration:nil URL:aURL options:nil error:&error];
        if (store == nil) {
            NSLog(@"Error creating changes persistent store: %@", error);
            return nil;
        }
    }
    return self;
}

@end
