//
//  SNKConflict.h
//  SyncKit
//
//  Created by Logan Collins on 7/2/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SyncKit/SyncKitDefines.h>


typedef SYNCKIT_ENUM(NSUInteger, SNKConflictType) {
    SNKConflictTypePropertyModified = 0,    // A property was modified on both the local and remote records
                                            // Allowed resolutions: ignore, edit, merge, delete, cancel
    
    SNKConflictTypePropertyRequired = 1,    // A required property is nil
                                            // Allowed resolutions: ignore, edit, delete, cancel
    
    SNKConflictTypeRecordOrphaned   = 2,    // A record with a required relationship has been orphaned
                                            // Allowed resolutions: ignore, edit, delete, cancel
};


typedef SYNCKIT_ENUM(NSUInteger, SNKConflictAction) {
    SNKConflictActionIgnore = 0,     // The conflict will be ignored, possibly causing the sync session to fail
    SNKConflictActionEdit,           // The record property was edited by the user or by the delegate to reflect a consistent state
    SNKConflictActionMerge,          // The record will be merged according to attribute timestamp
    SNKConflictActionDelete,         // The record will be deleted
    SNKConflictActionCancel,         // The sync session will be cancelled
};


@class SNKRecord;


@interface SNKConflict : NSObject

- (id)initWithType:(SNKConflictType)type localRecord:(SNKRecord *)localRecord remoteRecord:(SNKRecord *)remoteRecord;

@property (readonly) SNKConflictType type;
@property (strong, readonly) SNKRecord *localRecord;
@property (strong, readonly) SNKRecord *remoteRecord;

@property SNKConflictAction resolutionAction;
@property (strong) SNKRecord *resolvedRecord;

@end
