//
//  SNKCredentials.m
//  SyncKit
//
//  Created by Logan Collins on 4/8/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import "SNKCredentials.h"
#import "SNKCredentials_Private.h"


@implementation SNKCredentials {
    NSMutableDictionary *_storage;
    NSLock *_storageLock;
}

+ (BOOL)supportsSecureCoding {
    return YES;
}

- (id)init {
    self = [super init];
    if (self) {
        _storage = [NSMutableDictionary dictionary];
        _storageLock = [[NSLock alloc] init];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
    self = [super init];
    if (self) {
        self.type = [aDecoder decodeIntegerForKey:@"type"];
        
        _storage = [NSMutableDictionary dictionary];
        
        NSDictionary *userInfo = nil;
        if ([NSCoder instancesRespondToSelector:@selector(decodeObjectOfClass:forKey:)]) {
            userInfo = [aDecoder decodeObjectOfClass:[NSDictionary class] forKey:@"storage"];
        }
        else {
            userInfo = [aDecoder decodeObjectForKey:@"storage"];
        }
        
        if (userInfo != nil) {
            [_storage setDictionary:userInfo];
        }
    }
    return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder {
    [aCoder encodeInteger:self.type forKey:@"type"];
    
    [_storageLock lock];
    [aCoder encodeObject:_storage forKey:@"storage"];
    [_storageLock unlock];
}

- (id)copyWithZone:(NSZone *)zone {
    SNKCredentials *credentials = [[[self class] allocWithZone:zone] init];
    
    credentials.type = self.type;
    
    [_storageLock lock];
    credentials->_storage = [[NSMutableDictionary alloc] initWithDictionary:_storage copyItems:YES];
    [_storageLock unlock];
    
    return credentials;
}

- (NSString *)description {
    return [NSString stringWithFormat:@"%@ %@", [super description], _storage];
}

- (id)objectForKey:(NSString *)key {
    [_storageLock lock];
    id value = _storage[key];
    [_storageLock unlock];
    return value;
}

- (void)setObject:(id <NSCoding, NSCopying>)object forKey:(NSString *)key {
    [_storageLock lock];
    if (object != nil) {
        _storage[key] = object;
    }
    else {
        [_storage removeObjectForKey:key];
    }
    [_storageLock unlock];
}


#pragma mark -
#pragma mark OAuth

- (id)initWithOAuthToken:(NSString *)token tokenSecret:(NSString *)secret {
    self = [self init];
    if (self) {
        self.type = SNKCredentialsTypeOAuth;
        [self setObject:token forKey:@"OAuthToken"];
        [self setObject:secret forKey:@"OAuthTokenSecret"];
    }
    return self;
}

- (NSString *)OAuthToken {
    return [self objectForKey:@"OAuthToken"];
}

- (NSString *)OAuthTokenSecret {
    return [self objectForKey:@"OAuthTokenSecret"];
}

@end
